// Copyright (c) 2016, root ext. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of dna;

class DynamicLibrary<T> {
  final int STACK_SIZE = 256;

  Type type;
  String name;
  int libraryPointer;

  DynamicLibrary(){
    this.type = T;
    print('type $type');

    InstanceMirror im = reflectClass(type).metadata.first.getField(#name);
    this.name = im.reflectee;

    libraryPointer = nLoadLibrary(this.name);
    print('lib pointer $libraryPointer');
  }

  dynamic noSuchMethod(Invocation invocation){
    print('method ${MirrorSystem.getName(invocation.memberName)}');

    ClassMirror classMirror = reflectClass(type);
    MethodMirror mm = classMirror.instanceMembers.values.where((e)=> e.isRegularMethod && e.owner == classMirror && e.simpleName == invocation.memberName).first;

    List<TypedData> data = new List<TypedData>(invocation.positionalArguments.length);

    var methodPointer = nFindSymbol(libraryPointer, MirrorSystem.getName(invocation.memberName));
    print('method pointer $methodPointer');
    var vm = nCreateCallVM(STACK_SIZE);

    before(invocation, mm, vm, data);

    var result;
    if (mm.returnType == reflectClass(int)){
      result = nCallInt(vm, methodPointer);
    } else if (mm.returnType == reflectClass(bool)){
      result = nCallBool(vm, methodPointer);
    } else if (mm.returnType == reflectClass(String)){
      result = nCallString(vm, methodPointer);
    }else {
      nCallVoid(vm, methodPointer);
    }

    after(invocation, mm, data);

    nFreeCallVM(vm);
    return result;
  }

  void before(Invocation invocation, MethodMirror mm, int vm, List<TypedData> data){
    int index = 0;
    for (var p in mm.parameters){
      print('param ${MirrorSystem.getName(p.simpleName)}');
      for (var m in p.metadata.where((e)=> e.type == reflectClass(Arg))){
        if (m.getField(#type).reflectee == ARG_INT){
          nArgInt(vm, invocation.positionalArguments[index]);
        }
        if (m.getField(#type).reflectee == ARG_STRING){
          String str = invocation.positionalArguments[index].value;
          data[index] = new Uint8List(str.length+1);
          nArgTypedData(vm, data[index]);
        }
        if (m.getField(#type).reflectee == ARG_POINTER){
          data[index] = new Uint64List(1);
          nArgTypedData(vm, data[index]);
        }
        if (m.getField(#type).reflectee == ARG_LISTINT){
          var arg =invocation.positionalArguments[index];
          if (arg != null){
            data[index] = new Uint32List.fromList(arg.value);
            nArgTypedData(vm, data[index]);
          }
        }
      }
      index++;
    }
  }

  void after(Invocation invocation, MethodMirror mm, List<TypedData> data){
    int index = 0;
    for (var p in mm.parameters) {
      for (var m in p.metadata.where((e)=> e.type == reflectClass(Arg))){
        if (m.getField(#type).reflectee == ARG_STRING){
          Uint8List list = data[index];
          invocation.positionalArguments[index].value = new String.fromCharCodes(list);
        }
        if (m.getField(#type).reflectee == ARG_POINTER){
          Uint64List list = data[index];
          invocation.positionalArguments[index].value = list[0];
        }
        if (m.getField(#type).reflectee == ARG_LISTINT){
          var arg = invocation.positionalArguments[index];
          if (arg != null) {
            Uint32List list = data[index];
            arg.value = list.toList();
          }
        }
      }
      index++;
    }
  }
}

class Strings {
  static String fill(String char, int length){
    return new String.fromCharCodes(new List<int>.filled(length, char.codeUnitAt(0)));
  }
}

const int ARG_BYTE = 0;
const int ARG_SHORT = 1;
const int ARG_INT = 2;
const int ARG_LONG = 3;
const int ARG_LONGLONG = 4;
const int ARG_STRING = 5;
const int ARG_POINTER = 6;
const int ARG_LISTINT = 7;

class Arg {
  final int type;

  const Arg(this.type);
}

class Lib {
  final String name;

  const Lib(this.name);
}

class Ref<T> {
  T value;

  Ref(this.value);
}
